/* 
 * Copyright (C) 2002 by Greg Gardner
 * 
 * This file is part of elseed, a caller-id program for your TiVo.
 * 
 * elseed is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * elseed is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: elseed.c,v 1.1.1.1 2002/04/15 22:24:51 greg Exp $
 */


#include "elseed.h"
#include "osd.h"

/* loop til we die */
int STOP = FALSE; 

/* incoming caller id flag */
int INC_CID = FALSE;

/* the caller id structure */
caller_id cid;

/* the main shiznit */
int main (int argc, char *argv[])  {
    int fd, result;
    char inbuff[SZ_BUFF], outbuff[SZ_BUFF];
    char *sloc, *eloc;
    struct termios oldtio, newtio;
    struct stat statbuf;

    (void) signal (SIGINT, getout);	/* exit cleanly on ctrl-c */ 

    SetupTextOSD();

start:

    fd = open (MODEMDEVICE, O_RDWR | O_NOCTTY);
    if (fd < 0)  {
        perror (MODEMDEVICE); 
        exit (-1);
    }
    
    save_modem_settings(fd, &oldtio, &newtio);

    printf ("Initializing modem...\n");
    write (fd, MODEMINIT, strlen (MODEMINIT));

    memset (&outbuff, 0, SZ_BUFF);			/* init the buff */
    while (STOP == FALSE)  {			/* loop for input */
        if (wait_for_input(fd, 60 * 1000)) {
            sleep(1); /* For safe keeping so we don't get stuck in a loop */
            if (stat("/var/tmp/modemlock", &statbuf) == 0) {
                /* Tivo has a lock on the modem, don't 
                   do a read, and reopen the port from 
                   scratch after tivo has had its way with it */
                printf("Tivo's making a call, won't step on his toes\n");
                while (stat("/var/tmp/modemlock", &statbuf) == 0) {
                    sleep(10);
                }
                close(fd);
                printf("Restarting...\n");
                goto start;
            }
            while ((result = read(fd, inbuff, SZ_BUFF - strlen(outbuff) - 1)) > 0) {    /* grab the buff */
                inbuff[result] = 0;			/* mark end of buff */
                strcat (outbuff, inbuff);		/* build our str */
            }
            sloc = outbuff;
            while (sloc != NULL)  {	/* at end of str */
                if ((eloc = strstr (sloc, "\r\n")) != NULL) {
                     if (parse (sloc, eloc - sloc))  {   /* parse it out */
                         printit ();             /* dump output */
                         logit (LOG_FILE);       /* log output */
                         osdit();
                     }
                     sloc = eloc + 2;
                }
                else {
                    break;
                }
            }
            memset (outbuff, 0, SZ_BUFF);	/* reset the buff */
        }
    }

    tcsetattr(fd, TCSANOW, &oldtio);  /* set the port back */
    FreeTextOSD();
    printf ("Exited cleanly.\n");
    return (0);
}

void save_modem_settings(int fd, struct termios *oldtio, struct termios *newtio) {
    tcgetattr (fd, oldtio);  /* save current port settings */
	
    memset (newtio, 0, sizeof(*newtio));
    newtio->c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
    newtio->c_iflag = IGNPAR;
    newtio->c_oflag = 0;

    /* set input mode (non-canonical, no echo,...) */
    newtio->c_lflag = 0;

    newtio->c_cc[VTIME] = 50;	/* read after x tenths of a second */
    newtio->c_cc[VMIN] = 0;     /* read after x characters */

    tcflush (fd, TCIFLUSH);
    tcsetattr (fd, TCSANOW, newtio);
    return;
}

int wait_for_input(fd, msecs) {
    fd_set rfds;
    struct timeval timeout, *tptr;
    int ret;

    FD_ZERO( &rfds );
    FD_SET( fd, &rfds );
    if ( msecs >= 0 )
    {
        timeout.tv_sec = msecs / 1000;
        timeout.tv_usec = (msecs % 1000) * 1000;
        tptr = &timeout;
    }
    else
        tptr = NULL;

    ret = select( FD_SETSIZE, &rfds, NULL, NULL, tptr );
    return ret > 0;
}

/* parse it out to get the good stuff */
int parse (char *input, int len)  {
    int	ret_val = 0;
    char *dup;

    dup = malloc((len + 1)*sizeof(char));
    strncpy (dup, input, len);
    dup[len] = 0;

    /* printf ("Debug: parse: input = %s\n", input); */

    if (strncmp (dup, "RING", 4) == 0)  {
        printf ("Modem: RING\n");
    }

    if (strncmp (dup, "OK", 2) == 0)  {
        printf ("Modem: OK\n");
    }

    if (strncmp (dup, "ERROR", 5) == 0)  {
        printf ("Modem: ERROR\n");
    }

    /* w00t! caller id info coming in, flip the flag on */
    if (strstr (dup, CID_DATE) != NULL)  {
        memset (&cid, 0, sizeof (caller_id)); /* init struct */
        INC_CID = TRUE;
    }

    /* copy all strings from the end of their flag to null */
    if (INC_CID == TRUE)  {
        if (strstr (dup, CID_DATE) != NULL)  {
            strcpy (cid.date, dup + strlen (CID_DATE));
        }
        if (strstr (dup, CID_TIME) != NULL)  {
            strcpy (cid.time, dup + strlen (CID_TIME));
        }
        if (strstr (dup, CID_NAME) != NULL)  {
            strcpy (cid.name, dup + strlen (CID_NAME));
        }
        if (strstr (dup, CID_NMBR) != NULL)  {
            strcpy (cid.nmbr, dup + strlen (CID_NMBR));
            /* last entry, turn the flag off */
            INC_CID = FALSE;
            ret_val = TRUE;
        }
    }

    free (dup);
    return (ret_val);
}

void osdit() {
    int x, y, fg = 4, bg = 1;
    ClearOSD();
    x = 1; y = 11;
    DrawString(&x, &y, cid.name, fg, bg);
    x = 1; y = 12;
    DrawString(&x, &y, cid.nmbr, fg, bg);
    DrawOSD();
    sleep(10);
    ClearOSD();
    DrawOSD();
}

void printit ()  {
    printf ("--Call Logged--\n");
    printf ("  Date:   %s\n", cid.date);
    printf ("  Time:   %s\n", cid.time);
    printf ("  Name:   %s\n", cid.name);
    printf ("  Number: %s\n\n", cid.nmbr);
}

void logit (char *logfile)  {
    FILE	*file;
    time_t	unix_time;
    
    unix_time = time (NULL);

    if ((file = fopen (logfile, "a")) == NULL)  {
        printf ("Failed to write to %s.\n", logfile);
    }  else  {
        fprintf (file, "%s\t%s\t%s\t%s\t%ld\n",
                 cid.date,
                 cid.time,
                 cid.name,
                 cid.nmbr,
                 unix_time);
    }
    fclose (file);
}

/* let ya know we're exiting cleanly */
void getout (int sig)  {
    printf ("Caught signal %d, standby while exiting...\n", sig);
    STOP = TRUE;
}
